#include "bm1422.h"

bm1422_err_t bm1422_read(bm1422_t* dev, uint8_t reg, uint8_t* buf, size_t len)
{
  bm1422_err_t err = BM1422_OK;
  bool ackbit;
  i2c_t* i2c = dev->bus;
  i2c->start();
  i2c->write(dev->addr << 1);
  ackbit = i2c->write(reg);
  /* if(!ackbit) { */
  /*   err = BM1422_BUS_ERROR; */
  /*   i2c->stop(); */
  /*   goto exit; */
  /* } */
  i2c->stop();
  i2c->start();
  i2c->write(dev->addr << 1 | 1);
  for(size_t i=0 ; i < len - 1; ++i)
  {
    buf[i] = i2c->read(i2c->ack);
  }
  buf[len - 1] = i2c->read(i2c->nak);
  i2c->stop();
exit:
  return err;
}

bm1422_err_t bm1422_write(bm1422_t* dev, uint8_t reg, uint8_t* buf, size_t len)
{
  bm1422_err_t err = BM1422_OK;
  i2c_t* i2c = dev->bus;
  i2c->start();
  i2c->write(dev->addr << 1);
  i2c->write(reg);
  for(size_t i=0 ; i < len; ++i)
  {
    i2c->write(buf[i]);
  }
  i2c->stop();
  return err;
}

bm1422_err_t bm1422_setup(bm1422_t* dev, i2c_t* bus, uint8_t addr)
{
  bm1422_err_t err = BM1422_OK;
  dev->bus = bus;
  dev->addr = addr;
  uint8_t chip_id;
  err = bm1422_read(dev, BM1422_REG_WAI, &chip_id, 1);
  if(chip_id != 0x41)
  {
    err = BM1422_SETUP_ERROR;
    goto exit;
  }
exit:
  return err;
}

void bm1422_force(bm1422_t* dev)
{
  uint8_t v8 = 0x40;
  bm1422_write(dev, BM1422_REG_CNTL3, &v8, 1);
}

void bm1422_preamble(bm1422_t* dev, bm1422_mode_t mode, bm1422_bitdepth_t bd)
{
  uint8_t v8;
  uint16_t v16 = 0;
  switch(bd)
  {
  case BM1422_12BIT:
    v8 = 0x82 | BMI1422_ODR_1KHZ | mode;
    break;
  case BM1422_14BIT:
    v8 = 0x82 | BMI1422_ODR_1KHZ | mode;
    break;
  }
  bm1422_write(dev, BM1422_REG_CNTL1, &v8, 1);
  bm1422_write(dev, BM1422_REG_CNTL4, (uint8_t*)&v16, 2);
}

bool bm1422_drdy(bm1422_t* dev)
{
  uint8_t rv;
  bm1422_read(dev, BM1422_REG_STA1, &rv, 1);
  return rv > 0;
}

void bm1422_wait_drdy(bm1422_t* dev)
{
  for(;;) {
    if(bm1422_drdy(dev))
    {
      break;
    }
  }
}

bm1422_raw_t bm1422_single_shot_read(bm1422_t* dev)
{
  int16_t x, y, z;
  bm1422_read(dev, BM1422_REG_DATAX, (uint8_t*)&x, 2);
  bm1422_read(dev, BM1422_REG_DATAY, (uint8_t*)&y, 2);
  bm1422_read(dev, BM1422_REG_DATAZ, (uint8_t*)&z, 2);
  bm1422_raw_t res;
  res.x = x;
  res.y = y;
  res.z = z;
  return res;
}
