#pragma once

#include "p2-i2c.h"

#include <stdbool.h>
#include <sys/types.h>
#include <stdint.h>

#define BM1422_REG_WAI 0x0F
#define BM1422_REG_CNTL1 0x1B
#define BM1422_REG_CNTL2 0x1C
#define BM1422_REG_CNTL3 0x1D
#define BM1422_REG_CNTL4 0x5C
#define BM1422_REG_DATAX 0x10
#define BM1422_REG_DATAY 0x12
#define BM1422_REG_DATAZ 0x14
#define BM1422_REG_STA1 0x18

#define BMI1422_ODR_10HZ (0 << 3)
#define BMI1422_ODR_20HZ (2 << 3)
#define BMI1422_ODR_100HZ (1 << 3)
#define BMI1422_ODR_1KHZ (3 << 3)

typedef enum {
  BM1422_OK,
  BM1422_SETUP_ERROR,
  BM1422_BUS_ERROR,
} bm1422_err_t;

typedef enum {
  BM1422_CONTINUOUS = 0x00,
  BM1422_SINGLE_SHOT = 0x40,
} bm1422_mode_t;

typedef enum {
  BM1422_12BIT,
  BM1422_14BIT,
} bm1422_bitdepth_t;

typedef struct {
  i2c_t* bus;
  uint8_t addr;
} bm1422_t;

typedef struct {
  int16_t x;
  int16_t y;
  int16_t z;
} bm1422_raw_t;

#ifndef MAG_INLINE
#define MAG_INLINE
#endif

MAG_INLINE bm1422_err_t bm1422_read(bm1422_t* dev, uint8_t reg, uint8_t* buf, size_t len);
MAG_INLINE bm1422_err_t bm1422_write(bm1422_t* dev, uint8_t reg, uint8_t* buf, size_t len);
MAG_INLINE bm1422_err_t bm1422_setup(bm1422_t* dev, i2c_t* bus, uint8_t addr);
MAG_INLINE void bm1422_preamble(bm1422_t* dev, bm1422_mode_t mode, bm1422_bitdepth_t bd);
MAG_INLINE bool bm1422_drdy(bm1422_t* dev);
MAG_INLINE void bm1422_wait_drdy(bm1422_t* dev);
MAG_INLINE void bm1422_force(bm1422_t* dev);
MAG_INLINE bm1422_raw_t bm1422_single_shot_read(bm1422_t* dev);
