#pragma once

#include "p2-i2c.h"

#include <stdbool.h>
#include <stdint.h>
#include <sys/types.h>
#include <math.h>

#define BMI088_REG_ACC_ID 0x00
#define BMI088_REG_ACC_ERR 0x02
#define BMI088_REG_ACC_STATUS 0x03
#define BMI088_REG_ACC_DATA 0x12
#define BMI088_REG_ACC_PWR_CTRL 0x7D
#define BMI088_REG_ACC_SOFTRESET 0x7E
#define BMI088_REG_ACC_CONF 0x40
#define BMI088_REG_ACC_RANGE 0x41
#define BMI088_REG_GYR_ID 0
#define BMI088_REG_GYR_DATA 0x02
#define BMI088_REG_GYR_RANGE 0x0F
#define BMI088_REG_GYR_BANDWIDTH 0x10
#define BMI088_REG_GYR_SOFTRESET 0x14

typedef enum {
  BMI088_OK,
  BMI088_INIT_ERROR,
  BMI088_SELFTEST_ERROR,
  BMI088_ERROR_REG,
} bmi088_err_t;

typedef enum {
  BMI088_ACC_BWP_OSR4 = 0x08,
  BMI088_ACC_BWP_OSR2 = 0x09,
  BMI088_ACC_BWP_NORMAL = 0x0A
} bmi088_acc_bwp;

typedef enum {
  BMI088_ACC_ODR_12_5 = 0x05,
  BMI088_ACC_ODR_25 = 0x06,
  BMI088_ACC_ODR_50 = 0x07,
  BMI088_ACC_ODR_100 = 0x08,
  BMI088_ACC_ODR_200 = 0x09,
  BMI088_ACC_ODR_400 = 0x0A,
  BMI088_ACC_ODR_800 = 0x0B,
  BMI088_ACC_ODR_1600 = 0x0C,
} bmi088_acc_odr_t;

typedef enum {
  BMI088_ACC_RANGE_3G = 0x00,
  BMI088_ACC_RANGE_6G = 0x01,
  BMI088_ACC_RANGE_12G = 0x02,
  BMI088_ACC_RANGE_24G = 0x03,
} bmi088_acc_range_t;

typedef enum {
  BMI088_GYR_RANGE_2000S = 0x00,
  BMI088_GYR_RANGE_1000S = 0x01,
  BMI088_GYR_RANGE_500S = 0x02,
  BMI088_GYR_RANGE_250S = 0x03,
  BMI088_GYR_RANGE_125S = 0x04,
} bmi088_gyr_range_t;

typedef enum {
  BMI088_GYR_ODR_2000_532 = 0x00,
  BMI088_GYR_ODR_2000_230 = 0x01,
  BMI088_GYR_ODR_1000_116 = 0x02,
  BMI088_GYR_ODR_400_47 = 0x03,
  BMI088_GYR_ODR_200_23 = 0x04,
  BMI088_GYR_ODR_100_12 = 0x05,
  BMI088_GYR_ODR_200_64 = 0x06,
  BMI088_GYR_ODR_100_32 = 0x07,
} bmi088_gyr_odr_t;

typedef struct {
  int16_t acc[3];
  int16_t gyr[3];
} bmi088_data_raw_t;

typedef struct {
  float acc[3];
  float gyr[3];
} bmi088_data_t;


typedef struct {
  uint8_t acc_address;
  uint8_t gyr_address;
  float acc_factor;
  float gyr_factor;
  i2c_t *bus;
} bmi088_t;

#ifndef IMU_INLINE
#define IMU_INLINE
#endif

IMU_INLINE bmi088_err_t bmi088_read(bmi088_t* dev, uint8_t address, uint8_t reg, uint8_t* buf, size_t len);
IMU_INLINE bmi088_err_t bmi088_write(bmi088_t* dev, uint8_t address, uint8_t reg, uint8_t* buf, size_t len);
IMU_INLINE bool bmi088_selftest(bmi088_t* dev);
IMU_INLINE void bmi088_setup_samplerate(bmi088_t* dev, bmi088_acc_bwp acc_bwp, bmi088_acc_odr_t acc_odr, bmi088_gyr_odr_t gyr_odr);
IMU_INLINE bool bmi088_acc_drdy(bmi088_t* dev);
IMU_INLINE void bmi088_setup_range(bmi088_t* dev, bmi088_acc_range_t acc_range, bmi088_gyr_range_t gyr_range);
IMU_INLINE bmi088_data_t bmi088_convert_raw(bmi088_t* dev, bmi088_data_raw_t* data);
IMU_INLINE bmi088_err_t bmi088_setup(bmi088_t* dev, i2c_t* bus, uint8_t acc_address, uint8_t gyr_address);
