#include "ms5611.h"

void ms5611_start_temp_conversion(ms5611_t* dev, ms5611_cmd_d2_t d2_cmd)
{
  i2c_t* bus = dev->bus;
  bus->start();
  bus->write(dev->addr << 1);
  bus->write((uint8_t)d2_cmd);
  bus->stop();
}

void ms5611_start_pressure_conversion(ms5611_t* dev, ms5611_cmd_d1_t d1_cmd)
{
  i2c_t* bus = dev->bus;
  bus->start();
  bus->write(dev->addr << 1);
  bus->write((uint8_t)d1_cmd);
  bus->stop();
}

void ms5611_reset(ms5611_t* dev)
{
  i2c_t* bus = dev->bus;
  bus->start();
  bus->write(dev->addr << 1);
  bus->write(MS5611_CMD_RESET);
  bus->stop();
}

uint16_t ms5611_read_prom(ms5611_t* dev, uint8_t reg)
{
  uint16_t res;
  i2c_t* bus = dev->bus;
  bus->start();
  bus->write(dev->addr << 1);
  bus->write(MS5611_CMD_PROM_READ | (reg << 1));
  bus->stop();
  bus->start();
  bus->write(dev->addr << 1 | 1);
  res =bus->read(bus->ack) << 8;
  res |= bus->read(bus->nak);
  bus->stop();
  return res;
}

uint32_t ms5611_read_adc(ms5611_t* dev)
{
  uint32_t res;
  i2c_t* bus = dev->bus;
  bus->start();
  bus->write(dev->addr << 1);
  bus->write(MS5611_CMD_ADC_READ);
  bus->stop();
  bus->start();
  bus->write(dev->addr << 1 | 1);
  res =bus->read(bus->ack) << 16;
  res |= bus->read(bus->ack) << 8;
  res |= bus->read(bus->nak);
  bus->stop();
  return res;
}

float ms5611_read_temp(ms5611_t* dev)
{
  uint32_t adc = ms5611_read_adc(dev);
  float dt = (float)adc - dev->cal_data[4];
  dev->dt = dt;
  float temp = 2000.0 + dt * dev->cal_data[5];
  return temp / 100.0;
}

// Must be called *after* ms5611_read_temp, because
// only then we have the current dT
float ms5611_read_pressure(ms5611_t* dev)
{
  uint32_t adc = ms5611_read_adc(dev);
  float off = dev->cal_data[1] + (dev->cal_data[3] * dev->dt);
  float sens = dev->cal_data[0] + (dev->cal_data[2] * dev->dt);
  float pressure = ((float)adc * sens / 2097152.0 - off) / 32768.0;
  return pressure / 100.0;
}

ms5611_err_t ms5611_setup(ms5611_t* dev, i2c_t* bus, uint8_t addr)
{
  ms5611_err_t err = MS5611_OK;
  dev->addr = addr;
  dev->bus = bus;
  ms5611_reset(dev);
  _waitms(10); // just guessed...
  // Off by one, as the PROM seems to have 7 values
  // and the first is always 0.0
  // Also pre-compute all fixed calculations to reduce the amount
  // of ops later
  dev->cal_data[0] = (float)ms5611_read_prom(dev, 1) * 32768.0;
  dev->cal_data[1] = (float)ms5611_read_prom(dev, 2) * 65536.0;
  dev->cal_data[2] = (float)ms5611_read_prom(dev, 3) / 256.0;
  dev->cal_data[3] = (float)ms5611_read_prom(dev, 4) / 128.0;;
  dev->cal_data[4] = (float)ms5611_read_prom(dev, 5) * 256.0;
  dev->cal_data[5] = (float)ms5611_read_prom(dev, 6) / 8388608.0;
exit:
  return err;
}
