#pragma once

#include "p2-i2c.h"
#include <stdint.h>

#define MS5611_CMD_ADC_READ 0x00
#define MS5611_CMD_RESET 0x1E
#define MS5611_CMD_PROM_READ 0xA0

typedef enum {
  MS5611_OK,
  MS5611_SETUP_ERROR
} ms5611_err_t;

typedef enum {
  MS5611_CMD_D1_OSR_256=0x40,
  MS5611_CMD_D1_OSR_512=0x42,
  MS5611_CMD_D1_OSR_1024=0x44,
  MS5611_CMD_D1_OSR_2048=0x46,
  MS5611_CMD_D1_OSR_4096=0x48,
} ms5611_cmd_d1_t;

typedef enum {
  MS5611_CMD_D2_OSR_256=0x50,
  MS5611_CMD_D2_OSR_512=0x52,
  MS5611_CMD_D2_OSR_1024=0x54,
  MS5611_CMD_D2_OSR_2048=0x56,
  MS5611_CMD_D2_OSR_4096=0x58,
} ms5611_cmd_d2_t;

typedef struct {
  i2c_t* bus;
  uint8_t addr;
  float dt;
  float cal_data[6];
} ms5611_t;

#ifndef BARO_INLINE
#define BARO_INLINE
#endif

BARO_INLINE void ms5611_start_temp_conversion(ms5611_t* dev, ms5611_cmd_d2_t d2_cmd);
BARO_INLINE void ms5611_start_pressure_conversion(ms5611_t* dev, ms5611_cmd_d1_t d1_cmd);
BARO_INLINE void ms5611_reset(ms5611_t* dev);
BARO_INLINE uint16_t ms5611_read_prom(ms5611_t* dev, uint8_t reg);
BARO_INLINE uint32_t ms5611_read_adc(ms5611_t* dev);
BARO_INLINE float ms5611_read_temp(ms5611_t* dev);
// Must be called *after* ms5611_read_temp, because
// only then we have the current dT
BARO_INLINE float ms5611_read_pressure(ms5611_t* dev);
BARO_INLINE ms5611_err_t ms5611_setup(ms5611_t* dev, i2c_t* bus, uint8_t addr);
